/**
 * ATTT
 * (c) Kenta Cho (cs8k-cyu@asahi-net.or.jp)
 * http://www.asahi-net.or.jp/~cs8k-cyu/
 *
 * displaymanager.c(handle screen buffer)
 */

#include <PalmOS.h>
#include "attt.h"

WinHandle screenBufferH;
RectangleType sourceBounds;

/* bitmap handlers */
static MemHandle objectBitmapHandles[bitmapTypeCount];
BitmapPtr objectBitmapPtr[bitmapTypeCount];
static WinHandle objectWindowHandles[bitmapTypeCount];

/**
 * create off screen buffer
 */
void createScreenBuffer() {
  UInt16 error;
  WinHandle oldDrawWinH;

  screenBufferH = WinCreateOffscreenWindow
    (SCREEN_WIDTH, SCREEN_HEIGHT, screenFormat, &error);
  sourceBounds.topLeft.x = sourceBounds.topLeft.y = 0;
  sourceBounds.extent.x = SCREEN_WIDTH;
  sourceBounds.extent.y = SCREEN_HEIGHT;
  oldDrawWinH = WinGetDrawWindow();
  WinSetDrawWindow(screenBufferH);
  WinEraseRectangle(&sourceBounds, 0);
  WinSetDrawWindow(oldDrawWinH);
}

void clearDisplay() {
  WinEraseRectangle(&sourceBounds, 0);
}

static WinHandle oldDrawWinH;

void startDrawOffscreen() {
  oldDrawWinH = WinGetDrawWindow();
  WinSetDrawWindow(screenBufferH);
}

void endDrawOffscreen() {
  WinSetDrawWindow(oldDrawWinH);
}

/**
 * load bitmaps and hold these handlers
 */
void loadBitmaps() {
  int i;
  WinHandle oldDrawWinH;
  UInt16 error;
  oldDrawWinH = WinGetDrawWindow();
  for ( i=0; i<bitmapTypeCount; i++ ) {
    objectBitmapHandles[i] = DmGetResource(bitmapRsc, FIRST_BITMAP_ID+i);
    objectBitmapPtr[i] = MemHandleLock(objectBitmapHandles[i]);
    objectWindowHandles[i] = WinCreateOffscreenWindow
      (objectBitmapPtr[i]->width, objectBitmapPtr[i]->height,
       screenFormat, &error);
    ErrFatalDisplayIf(error, "Error loading images");
    WinSetDrawWindow(objectWindowHandles[i]);
    WinDrawBitmap(objectBitmapPtr[i], 0, 0);
  }
  WinSetDrawWindow(oldDrawWinH);
}

/**
 * relase bitmap handlers
 */
void unloadBitmaps() {
  int i;
  for ( i=0; i<bitmapTypeCount; i++ ) {
    MemPtrUnlock(objectBitmapPtr[i]);
    DmReleaseResource(objectBitmapHandles[i]);
    if (objectWindowHandles[i]) 
      WinDeleteWindow(objectWindowHandles[i], false);
  }
}

void drawThickLine(int sx, int sy, int dx, int dy) {
  Boolean hr = FALSE;
  if ( abs(sx-dx) < abs(sy-dy) ) {
    hr = TRUE;
  }
  WinDrawLine(sx, sy, dx, dy);
  if ( hr ) {
    WinDrawLine(sx-1, sy, dx-1, dy);
    WinDrawLine(sx+1, sy, dx+1, dy);
  } else {
    WinDrawLine(sx, sy-1, dx, dy-1);
    WinDrawLine(sx, sy+1, dx, dy+1);
  }
}

/*
 * set hand data to buffer
 */
static int xMin[SCREEN_HEIGHT], xMax[SCREEN_HEIGHT];

static void scanEdge(int x1, int y1, int x2, int y2) {
  int i;
  int l = abs(y2 -y1) + 1;
  long addx = (((long)x2 - x1) * 0xff) / l;
  long addy = (((long)y2 - y1) * 0xff) / l;
  long x = (long)x1 * 0xff;
  long y = (long)y1 * 0xff;
  for ( i=0 ; i<l ; i++, x+=addx, y+=addy ) {
    int py = y>>8;
    int px = x>>8;
    if ( py<0 || py>=SCREEN_HEIGHT ) continue;
    if ( xMin[py] > px ) {
      xMin[py] = px;
    }
    if ( xMax[py] < px ){
      xMax[py] = px;
    }
  }
}

UInt8 ptn[SCREEN_WIDTH*SCREEN_HEIGHT];

void initPtnBuf() {
  int i;
  for ( i=0 ; i<SCREEN_WIDTH*SCREEN_HEIGHT ; i++ ) {
    ptn[i] = 0;
  }
}

static void drawLineOnBuf(int min, int max, int y, int bufDat) {
  int i;
  for ( i=min ; i<=max ; i++ ) {
    ptn[i+y*SCREEN_WIDTH] = bufDat;
  }
}

int checkDot(int x, int y) {
  return ptn[x+y*SCREEN_WIDTH];
}

void drawPolygon(Pos p[], int num, int bufDat) {
  int i;
  int yMin = 9999, yMax = -9999;
  for ( i=0 ; i<SCREEN_HEIGHT ; i++ ) {
    xMin[i] = SCREEN_WIDTH-1;
    xMax[i] = 0;
  }
  for ( i=0 ; i<num ; i++ ) {
    if ( i == 0 ) {
      scanEdge(p[num-1].x, p[num-1].y, p[0].x, p[0].y);
    } else {
      scanEdge(p[i-1].x, p[i-1].y, p[i].x, p[i].y);
    }
    if ( p[i].y > yMax ) yMax = p[i].y;
    if ( p[i].y < yMin ) yMin = p[i].y;
  }
  for ( i=yMin ; i<yMax ; i++ ) {
    int mn = xMin[i], mx = xMax[i];
    if ( mn < 0 ) mn = 0;
    if ( mx >= SCREEN_WIDTH ) mx = SCREEN_WIDTH-1;
    drawLineOnBuf(mn, mx, i, bufDat);
  }
}
