/**
 * ATTT
 * (c) Kenta Cho (cs8k-cyu@asahi-net.or.jp)
 * http://www.asahi-net.or.jp/~cs8k-cyu/
 *
 * attt.c(game manager)
 */

#include <PalmOS.h>
#include "attt.h"

#define ATSCR_SIZE 16*10

static GameStatus gameStatus;

/*
 * draw title 
 */
static Boolean atScr[ATSCR_SIZE];
static int score = -1;
static int atPct;

void startTitle() {
  gameStatus.status = TITLE;
  if ( score == -1 ) {
    WinDrawChars("ATTT", 4, 68, 60);
  } else {
    int i;
    int x=0, y=25;
    WinDrawChars("You're", 6, 65, 15);
    for ( i=0 ; i<score ; i++ ) {
      if ( atScr[i] ) {
	WinDrawChars("", 2, x, y);
      } else {
	WinDrawChars("", 2, x, y);
      }
      x += 10;
      if ( x>=160 ) {
	x = 0; y += 10;
      }
    }
    WinDrawChars("shock!", 6, 65, 125);
  }
}

void gotoTitle() {
  gameStatus.status = TITLE;
  FrmGotoForm(TITLE_FORM);
}

/*
 * stage hand data
 */
#define STAGE_NUM 3

static StageData stageData[STAGE_NUM] = {
  {{80,159, 0,159, 10,20, 
    80,159, 10,20, 150,20, 
    80,159, 159,159, 150,20, -1},
   {60,159, 10,20, 80,120, 150,20, 100,159, 80,159, -1},
   {0,1,2,3,4,-1},
   {0,1,2,5, 5,2,3,4, -1}},

  {{80,159, 0,159, 15,80,
    80,159, 15,80, 40,40, 
    80,159, 40,40, 80,30,
    80,159, 80,30, 120,40,
    110,159, 120,40, 145,110, 
    110,159, 150,115, 159,159, -1},
   {45,159, 40,125, 10,85, 20,75, 54,111, 33,42, 47,38, 70,100, 72,30,
    88,30, 90,100, 113,38, 127,42, 110,111, 120,130, 140,105, 
    150,115, 130,159, 80,159, 100,159, -1},
   {0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,-1},
   {0,1,4,18, 1,2,3,4, 4,7,10,18, 4,5,6,7, 7,8,9,10, 10,13,14,18,
    10,11,12,13, 14,17,19,18, 14,15,16,17, -1}},

  {{80,159, 0,159, 12,113, 
    80,159, 28,97, 44,84, 
    80,159, 66,76, 94,76,
    80,159, 116,84, 132,97,
    80,159, 148,113, 159,159, -1},
   {30,159, 12,113, 28,97, 48,114, 44,84, 66,76, 80,105, 94,76, 116,84,
    112,114, 132,97, 148,113, 130,159, 80,159, -1},
   {0,1,2,3,4,5,6,7,8,9,10,11,12,-1},
   {0,1,2,3, 3,4,5,6, 6,7,8,9, 9,10,11,12, 0,3,6,13, 13,6,9,12, -1}},
};

#define HAND_AREA_NUM 99

static int areaNum;

static void drawStage(int stNum) {
  StageData *st = &(stageData[stNum]);
  int i = 0;
  areaNum = 0;
  while ( st->hitArea[i] != -1 ) {
    Pos pos[3];
    int j;
    for ( j=0 ; j<3 ; j++ ) {
      pos[j].x = st->hitArea[i]; i++;
      pos[j].y = st->hitArea[i]; i++;
    }
    areaNum++;
    drawPolygon(pos, 3, areaNum);
  }
  i = 0;
  while ( st->handLineCnt[i+1] != -1 ) {
    drawThickLine
      (st->handPoint[st->handLineCnt[i]*2], 
       st->handPoint[st->handLineCnt[i]*2+1],
       st->handPoint[st->handLineCnt[i+1]*2],
       st->handPoint[st->handLineCnt[i+1]*2+1]);
    i++;
  }
  i = 0;
  while ( st->handClsCnt[i] != -1 ) {
    Pos pos[4];
    int j;
    for ( j=0 ; j<4 ; j++ ) {
      pos[j].x = st->handPoint[st->handClsCnt[i]*2];
      pos[j].y = st->handPoint[st->handClsCnt[i]*2+1];
      i++;
    }
    drawPolygon(pos, 4, HAND_AREA_NUM);
  }
}

/*
 * draw tap position marker
 */
static void drawMarker(int clrMn, int setMn) {
  int i;
  for ( i=0 ; i<SCREEN_WIDTH*SCREEN_HEIGHT ; i += 31 ) {
    UInt8 pt = ptn[i];
    if ( pt == clrMn ) {
      Coord x = i%SCREEN_WIDTH, y = i/SCREEN_WIDTH;
      WinEraseLine(x, y, x, y);
    } else if ( pt == setMn ) {
      Coord x = i%SCREEN_WIDTH, y = i/SCREEN_WIDTH;
      WinDrawLine(x, y, x, y);
    }
  }
}

/*
 * draw a/ta letter
 */
MoveLetter atLetter;

static int atSize[] = {20,15,10, 20,15,10};

static void drawATLetter() {
  atLetter.x += atLetter.mx;
  atLetter.y += atLetter.my;
  atLetter.cnt--;
  if ( atLetter.spc!=0 && atLetter.spc!=3 ) atLetter.spc--; 
}

static void setATLetter(int x, int y, int spc) {
  int d = getDeg(x-80, y-160);
  atLetter.x = x<<4; atLetter.y = y<<4;
  atLetter.mx = sintbl[d]>>2; atLetter.my = -costbl[d]>>2; 
  atLetter.cnt = 10;
  atLetter.spc = spc*3+2;
}

/*
 * draw blood
 */
#define BLOOD_NUM 16

Blood blood[BLOOD_NUM];
int bspX, bspY, bspRad;

static void initBlood() {
  int i;
  for ( i=0 ; i<BLOOD_NUM ; i++ ) {
    blood[i].x = -9999;
  }
  bspRad = -1;
}

static void drawBlood() {
  int i;
  for ( i=0 ; i<BLOOD_NUM ; i++ ) {
    if ( blood[i].x != -9999 ) {
      Blood *bl = &(blood[i]);
      bl->px = bl->x+(bl->y&1)*2-1; bl->py = bl->y+(bl->x&1)*2-1;
      bl->x += bl->mx; bl->y += bl->my;
      WinDrawLine(bl->px, bl->py, bl->x, bl->y);
      bl->my += 2;
      if ( bl->px<0 || bl->px>SCREEN_WIDTH || bl->py>SCREEN_HEIGHT ) {
	bl->x = -9999;
      }
    }
  }
}

static void setBlood(int x, int y) {
  int i;
  for ( i=0 ; i<BLOOD_NUM ; i++ ) {
    if ( blood[i].x == -9999 ) {
      Blood *bl = &(blood[i]);
      bl->x = x; bl->y = y;
      //bl->mx = randN(12)-6; bl->my = randN(20)-16;
      bl->mx = randN(24)-12; bl->my = randN(40)-32;
      return;
    }
  }
}

static void drawBloodSpot() {
  RectangleType rct;
  rct.extent.x = rct.extent.y = bspRad;
  rct.topLeft.x = bspX + randN(32)-16;
  rct.topLeft.y = bspY + randN(32)-16;
  WinDrawRectangle(&rct, rct.extent.x/2);
}

/*
 * manage game status
 */
static int mrkNum, mnMv;
static int stage;

void startButtonPushed(int stg) {
  stage = stg;
  initGame();
  FrmGotoForm(GAME_FORM);
}

void startGame() {
  startDrawOffscreen();
  clearDisplay();
  drawStage(stage);
  WinDrawChars("܂", 6, 65, 0);
  endDrawOffscreen();
  gameStatus.status = READY;
  gameStatus.cnt = 20;
}

void initGame() {
  mrkNum = 0; mnMv = 1;
  score = 0;
  atPct = 0;
  initPtnBuf();
  initBlood();
  atLetter.cnt = -1;
}

static void clearMessage() {
  RectangleType rct;
  rct.topLeft.x = 40;
  rct.topLeft.y = 0;
  rct.extent.x = 80;
  rct.extent.y = 16;
  WinEraseRectangle(&rct, 0);
}

static int hitStat = 0;
static int penX, penY;

void penDowned(int x, int y) {
  if ( gameStatus.status == GAME_IN_PROGRESS ) {
    penX = x; penY = y;
    hitStat = checkDot(x, y);
  }
}

#define GAME_SEC 10

static int bldX, bldY;

void handleGameStatus() {
  if ( gameStatus.status == TITLE ) return;
  startDrawOffscreen();
  switch ( gameStatus.status ) {
  case GAME_IN_PROGRESS:
    gameStatus.cnt--; 
    if ( gameStatus.cnt <= 0 ) {
      gameStatus.status = TIMEUP;
      WinDrawChars("܂", 8, 60, 0);
    }
    if ( hitStat != 0 ) {
      if ( hitStat == mrkNum+1 ) {
	int prvMn = mrkNum;
	mrkNum += mnMv;
	if ( mrkNum >= areaNum ) {
	  mnMv = -1; mrkNum = areaNum-2;
	} else if ( mrkNum < 0 ) {
	  mnMv = 1; mrkNum = 1;
	}
	drawMarker(prvMn+1, mrkNum+1);
	if ( randN(16) < atPct ) {
	  atScr[score] = FALSE;
	  atPct -= randN(3);
	  setATLetter(penX, penY, 1);
	} else {
	  atScr[score] = TRUE;
	  atPct = 16;
	  setATLetter(penX, penY, 0);
	}
	if ( score == 0 ) clearMessage();
	score++; if ( score >= ATSCR_SIZE ) score = ATSCR_SIZE-1; 
      } else if ( hitStat == HAND_AREA_NUM ) {
	gameStatus.status = MISS;
	gameStatus.cnt = 0;
	bldX = penX; bldY = penY;
      }
      hitStat = 0;
    }
    break;
  case READY:
    gameStatus.cnt--;
    if ( gameStatus.cnt < 0 ) {
      gameStatus.status = GAME_IN_PROGRESS;
      gameStatus.cnt = 15*(100/INTERVAL);
      clearMessage();
      WinDrawChars("͂!", 7, 64, 0);
      drawMarker(-1, mrkNum+1);
    }
    break;
  case MISS:
    if ( randN(4) ) setBlood(bldX, bldY);
    drawBlood();
    if ( bspRad < 5 ) {
      if ( randN(12) ) {
	bspX = bldX + randN(100)-50; 
	bspY = bldY + randN(100)-50; 
	bspRad = randN(16)+24;
      }
    } else {
      drawBloodSpot();
      bspRad -= bspRad>>2;
    }
  case TIMEUP:
    gameStatus.cnt--;
    if ( gameStatus.cnt < -30 ) gotoTitle();
    break;
  }
  endDrawOffscreen();
  /* flip display */
  WinCopyRectangle
    (screenBufferH, NULL, &sourceBounds, 0, 0, winPaint);
  if ( atLetter.cnt > 0 ) {
    WinDrawBitmap(objectBitmapPtr[atLetter.spc], 
		  (atLetter.x>>4)-atSize[atLetter.spc],
		  (atLetter.y>>4)-atSize[atLetter.spc]);
    drawATLetter();
  }
}

